import requests
from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.signing import Signer
from django.http import JsonResponse
from django.utils.deprecation import MiddlewareMixin

from .utils import decrypt_data

signer = Signer()

User = get_user_model()

class ParseCodeDataMiddleware(MiddlewareMixin):
    
    def __call__(self, request):
        code = request.COOKIES.get('code')
        my_code_mod = None

        if (not request.user.is_authenticated and 
            code and 'set-code/' not in request.path):    

            my_code = request.COOKIES.get('my_code')
            try:
                my_code = signer.unsign_object(my_code)
                username = my_code['username']
                my_code = code == my_code['code'] 
            except:
                my_code = None

            if not my_code:
                sso_server = settings.SSO_SERVER_SESSIONID
                response = requests.get(
                    url=sso_server,
                    params={
                        'code':code,
                        'application_id':settings.APPLICATION_ID
                    }
                )

                response_json = response.json()
                if not response.ok:
                    response = JsonResponse(response_json, status=response.status_code)
                    response.delete_cookie('code')
                    response.delete_cookie('my_code')
                    return response
                
                response_json = response_json['code']
                username = decrypt_data(key=settings.KEY, ciphertext=response_json)['username']
                
                my_code_mod = signer.sign_object({
                    'username' : username,
                    'code' : code,   
                }) 

            try:
                request.user = User.objects.filter(username=username).first()
            except:
                response = JsonResponse({
                    'message' : "user is not found."
                    }, status=400)
                response.delete_cookie('code')
                response.delete_cookie('my_code')
                return response

        response = self.get_response(request)
        if my_code_mod:
            response.set_cookie('my_code', my_code_mod, expires=30)
        return response



import json
from base64 import b64decode

from cryptography.fernet import Fernet

def decrypt_data(key, ciphertext):
    fernet = Fernet(b64decode(key) )
    data = fernet.decrypt(ciphertext).decode()
    return json.loads(data)



import json

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.signing import Signer
from django.http import HttpResponseRedirect, JsonResponse
from django.shortcuts import redirect
from django.urls import reverse
from django.utils import timezone
from django.utils.dateparse import parse_datetime
from django.views import View

from .utils import decrypt_data

User = get_user_model()
signer = Signer()

def profile(request):
    if not request.user.is_authenticated:
        return JsonResponse({
            'user_data':None
        }, status=401)
    
    user = request.user
    return JsonResponse({
        'user_data':{
            'id': user.id,
            'username': user.username,
            'is_active': user.is_active,
            'is_staff': user.is_staff,
            'date_joined': user.date_joined,
            'last_login': user.last_login,
        }
    })
    
def login(request):
    next_url = request.GET.get('next') or '/'
    if request.user.is_authenticated:        
        return redirect(next_url)

    signed_obj = signer.sign_object({
        'date_time' : str(timezone.now()),
        'secret_key' : request.build_absolute_uri('/'),   
    })

    url_path = request.build_absolute_uri(reverse('set-code'))
    
    url = f'{settings.SSO_SERVER_LOGIN}?next={url_path}&frontend_url={next_url}&verify_code={signed_obj}&application_id={settings.APPLICATION_ID}'
    return redirect(url)

def logout(request):
    if not request.user.is_authenticated:
        return JsonResponse({'message':'You have not login to this sytem.'})

    next_url = request.GET.get('next') or request.build_absolute_uri('/')
    url = f'{settings.SSO_SERVER_LOGOUT}?next={next_url}'
    response = redirect(url)
    response.delete_cookie('my_code')
    response.delete_cookie('code')
    return response

class SetCodeView(View):
    def create_or_update_user(self, user_data):
        try:
            user = User.objects.get(username=user_data['username'])
            user.last_login = user_data.get('last_login', timezone.now())
            user.save()
        except:
            user = User.objects.create(
                username=user_data.get('username'),
                is_staff = user_data.get('is_staff', False),
                is_active = user_data.get('is_active', False),
                is_superuser = user_data.get('is_superuser', False),
                date_joined = user_data.get('created_at', timezone.now()),
                last_login = user_data.get('last_login', timezone.now()),
            )

        if hasattr(user, 'sso_id') and user_data.get('id'):
            user.sso_id = user_data.get('id')
            user.save()

        return user

    def get(self, request):

        verify_code = request.GET.get('verify_code')
        if not verify_code:
            return JsonResponse({
                'verify_code':'This field is required.',
            }, status=400)
        
        try:
            obj = signer.unsign_object(verify_code)
        except:
            return JsonResponse({
                'message':'Problems while decrypt data.',
            }, status=401)
        
        try:
            date_time = parse_datetime(obj['date_time'])
        except:
            return JsonResponse({
                'message':'Cannot parse time while decrypt data.',
            }, status=401)
        
        try:
            secret_key = parse_datetime(obj['secret_key'])
        except:
            return JsonResponse({
                'message':'Cannot parse secret_key while decrypt data.',
            }, status=401)
        
        if secret_key == request.build_absolute_uri('/'):
            return JsonResponse({
                'message':'Request is not origined from thios site.',
            }, status=401)
        
        diff_time = (timezone.now() - date_time).total_seconds()
        if diff_time > 60:
            return JsonResponse({
                    'message':'Token is expiry or invalid.',
                }, status=401)
        
        code = request.GET.get('code')
        if not code:
            return JsonResponse({
                'code':'This field is required.',
            }, status=400)
        
        plain_code = decrypt_data(key=settings.KEY, ciphertext=code)

        signed_obj = plain_code['signed_obj']
        user_data = json.loads(plain_code['user_data'])
        user = self.create_or_update_user(user_data)
        
        my_code = signer.sign_object({
            'username' : user.username,
            'code' : code,   
        }) 
        
        frontend_url = request.GET.get('frontend_url') or '/'
        response = HttpResponseRedirect(frontend_url)
        response.set_cookie('my_code', my_code, expires=30)
        response.set_cookie('code', signed_obj)
        return response



from django.urls import path

from .views import login, logout, profile, SetCodeView

urlpatterns = [
    path('admin/login/', login, name='login'),
    path('admin/logout/', logout, name='logout'),
    path('profile/', profile),
    path('set-code/', SetCodeView.as_view(), name='set-code'),
]



import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'readme.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-sso-client-manager',
    version='0.0.2',
    packages=['sso_client_manager'],
    include_package_data=True,
    license='MIT License',
    description='A Django app for the managing sso client that have login, logout and set code a in cookies.',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/rjnp2/django-sso-client-manager',
    author='rjnp2',
    author_email='rjnp2@outlook.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    python_requires=">=3.10",
    install_requires=[
        'cryptography>=1.0.1',
        'requests>=2.31.0',
    ],
    extras_require={
        "dev": ['twine>=4.0.2',]
    }
)
